/*------------------------------------------------*/
/* 04shaders/trojuhelnik/scale.c                  */

#define GLEW_STATIC
#include <GL/glew.h>
#include <stdio.h>
#include <GLFW/glfw3.h>
#include "linmath-extended.h"
#include "vertex-data.h"
#include "shader.h"
#include "events.h"

void initVariables(Shader *program);
void initBuffers(Shader *program);
void uninitBuffers();
void ticking(Shader * program, GLFWwindow* window);
void draw(Shader *program, GLFWwindow* window, GLuint VAO);
void animate();

GLuint VAO, BO[2];
struct {
	GLuint aVertexPositionLoc;
	GLuint aVertexColorLoc;
	GLuint uPMatrixLoc;
	GLuint uMVMatrixLoc;
} variables;

int main(void)
{
	GLFWwindow* window;
	/* Initialize the library */
	if (!glfwInit()) {
		return -1;
	}
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 0);
	//glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	/* Create a windowed mode window and its OpenGL context */
	window = glfwCreateWindow(640, 480, "04 - scale", NULL, NULL);
	if (!window)
	{
		printf("Nelze inicializovat kontext\n");
		glfwTerminate();
		return -1;
	}
	/* Make the window's context current */
	glfwMakeContextCurrent(window);
	printf("%s\n", glGetString(GL_VERSION));

	/* Load modern OpenGL functions */
	glewExperimental = GL_TRUE;
	glewInit();

	/* events */
	glfwSetKeyCallback(window, key_callback);
	glfwSetMouseButtonCallback(window, mouse_button_callback);

	//glfwSwapInterval(1);

	Shader program = shader("default.vs", "default.fs");
	if(!program.Program) {
		return -1;
	}

	initVariables(&program);
	initBuffers(&program);

	// setup GL state
	glEnable(GL_DEPTH_TEST);

	ticking(&program, window);
	uninitBuffers();

	glfwDestroyWindow(window);
	glfwTerminate();
	return 0;
}

void initVariables(Shader *program)
{
	variables.aVertexPositionLoc = program->getAttribLocation(program, "aVertexPosition");
	variables.aVertexColorLoc = program->getAttribLocation(program, "aVertexColor");
	variables.uMVMatrixLoc = program->getUniformLocation(program, "uMVMatrix");
	variables.uPMatrixLoc = program->getUniformLocation(program, "uPMatrix");
}

void initBuffers(Shader *program)
{
	glGenVertexArrays(1, &VAO);
	glGenBuffers(2, BO);

	glBindBuffer(GL_ARRAY_BUFFER, BO[0]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);

	glBindBuffer(GL_ARRAY_BUFFER, BO[1]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(colors), colors, GL_STATIC_DRAW);

	glBindVertexArray(VAO);
	{
		glBindBuffer(GL_ARRAY_BUFFER, BO[0]);
		glVertexAttribPointer(variables.aVertexPositionLoc, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(0);
	
		glBindBuffer(GL_ARRAY_BUFFER, BO[1]);
		glEnableVertexAttribArray(1);
		glVertexAttribPointer(variables.aVertexColorLoc, 4, GL_FLOAT, GL_FALSE, 4 * sizeof(GLfloat), (GLvoid*)0);
	}
	glBindVertexArray(0); // Unbind VAO
	glBindBuffer(GL_ARRAY_BUFFER, 0);
}

void uninitBuffers()
{
	glDeleteVertexArrays(1, &VAO);
	glDeleteBuffers(2, BO);
}

void ticking(Shader * program, GLFWwindow* window) {
	/* Loop until the user closes the window */
	while (!glfwWindowShouldClose(window))
	{
		animate();
		draw(program, window, VAO);
		glfwSwapBuffers(window);
		glfwPollEvents();
	}
}

void animate() {
}

void draw(Shader *program, GLFWwindow* window, GLuint VAO)
{
	int width, height;
	glfwGetFramebufferSize(window, &width, &height);
	glViewport(0, 0, width, height);

	glClearColor(1.f, 1.f, 1.f, 1.f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	program->Use(program);

	mat4x4 uMVMatrix, uPMatrix;
	mat4x4_identity(uMVMatrix);
	mat4x4_identity(uPMatrix);
	/* 1) Scale */
	 mat4x4_scale_aniso(uMVMatrix, uMVMatrix, 0.5, 0.5, 0.5);
	// mat4x4_print(uMVMatrix);
	/* 2) Translate + Scale */
	// mat4x4_translate(uMVMatrix, 1.0, 0., 0.);
	// mat4x4_scale_aniso(uMVMatrix, uMVMatrix, 0.5, 0.5, 0.5);
	/* 3) Scale + Translate */
	// mat4x4_scale_aniso(uMVMatrix, uMVMatrix, 0.5, 0.5, 0.5);
	// mat4x4_translate_in_place(uMVMatrix, 1.0, 0., 0.);

	glUniformMatrix4fv(variables.uMVMatrixLoc, 1, GL_FALSE, (const GLfloat *) uMVMatrix);
	glUniformMatrix4fv(variables.uPMatrixLoc, 1, GL_FALSE, (const GLfloat *) uPMatrix);

	// Draw container
	glBindVertexArray(VAO);
	glDrawArrays(GL_TRIANGLES, 0, sizeof(vertices)/sizeof(vertices[0])/3);
	glBindVertexArray(0);
}

/*------------------------------------------------*/
