#version 330 core

uniform vec3 uAmbientColor;
uniform vec3 uLightingPosition;
uniform vec3 uDirectionalColor;
uniform bool uLightBothSides;
uniform bool uUseSpecular;
uniform bool uUseDirectional;
 
in vec4 mvPosition;
in vec4 vColor;
in vec3 transformedNormal;

void main(void) {
	vec3 vLightIntenzity = uAmbientColor;
 
        vec3 normal = normalize(transformedNormal);
        vec3 lightDirection = normalize(uLightingPosition - mvPosition.xyz);

        // directional light
        if(uUseDirectional) {
                float lightIntenzity = dot(normal, lightDirection);
                if(uLightBothSides) {
                        lightIntenzity = max(lightIntenzity, -lightIntenzity);
                } else {
                        lightIntenzity = max(lightIntenzity, 0.0);
                }
                vLightIntenzity += uDirectionalColor * lightIntenzity;
        }
 
        // specular light
        if(uUseSpecular) {
                vec3 eyeDirection = normalize(-mvPosition.xyz);
                vec3 reflectionDirection = reflect(-lightDirection, normal);
                float specularLightIntenzity = max(dot(reflectionDirection, eyeDirection),0.0);
                specularLightIntenzity = pow(specularLightIntenzity,10.0);
                vLightIntenzity += uDirectionalColor * specularLightIntenzity;
        }
 
        gl_FragColor = vec4(vColor.rgb * vLightIntenzity, vColor.a);
}
